#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "srv.h"

bool test = 0;

int old, new;

char *usage =
"Usage:\n"
"  runlevelchange [--test] old new\n"
"old is one of N12345\n"
"new is one of 0123456";

static int whattodo (const struct service *service)
{
	int ret = 0;
	if (old != -1 && service->runlevel[old]) ret--;
	if (service->runlevel[new]) ret++;
	return ret;
}

static int sortorder (const void *v1, const void *v2)
{
	const struct service *s1 = v1;
	const struct service *s2 = v2;
	int i1, i2;
	i1 = whattodo(s1);
	i2 = whattodo(s2);
	if (i1 != i2) return i1 - i2;
	if (s1->sequence != s2->sequence) {
		if (i1 < 0) return s2->sequence - s1->sequence;
		else return s1->sequence - s2->sequence;
	}
	if (i1 < 0) return strcmp(s2->name, s1->name);
	else return strcmp(s1->name, s2->name);
}

int main (int argc, char *argv[])
{
	int i;
	struct service *service;
	struct services services;

	if (argc == 4 && !strcmp(argv[1], "--test")) {
		test = 1;
		argv++;
		argc--;
	}

	if (argc != 3 || strlen(argv[1]) != 1 || strlen(argv[2]) != 1 ||
	  strchr("N12345", argv[1][0]) == NULL || strchr("0123456", argv[2][0]) == NULL) {
		message(usage);
		exit(1);
	}

	if (argv[1][0] == 'N') old = -1;
	else old = argv[1][0] - '0';
	new = argv[2][0] - '0';

	if (services_read_list(&services) < 0) exit(1);
	service = malloc(services.nr * sizeof(struct service));
	for (i = 0; i < services.nr; i++) {
		service_read(services.name[i], &service[i]);
	}

	qsort(service, services.nr, sizeof(struct service), sortorder);

	for (i = 0; i < services.nr; i++) {
		switch (whattodo(&service[i])) {
		  case -1:
			if (test) printf("%s stop\n", service[i].name);
			else service_stop(&service[i]);
			break;
		  case 1:
			if (test) printf("%s start\n", service[i].name);
			else service_start(&service[i]);
			break;
		}
	}
	return 0;
}
