#ifndef SERVICE_H
#define SERVICE_H

#include <stdio.h>
#include <sys/types.h>

#define DEFAULTDIR "/etc/runlevel.d/default"
#define CUSTOMDIR "/etc/runlevel.d/custom"

typedef int bool;

struct services {
	char **name;
	int nr;
};

struct langvalue {
	char *lang;
	char *value;
};

struct localestring {
	struct langvalue *langvalues;
	int nr;
};

struct service {
	char *name;
	char *script;
	bool supportsreload_set;
	bool supportsreload;
	bool runlevel_set;
	bool runlevel[7];
	bool sequence_set;
	int sequence;
	bool uid_set;
	uid_t uid;
	bool gid_set;
	gid_t gid;
	bool restartseconds_set;
	int restartseconds;
	struct localestring *summary;
	struct localestring *description;
	bool nice_set;
	int nice;
};

struct valid_keyword {
	char *name;
	bool is_localestring;
	int (*add_fn) (const char *, const char *, struct service *);
	int (*remove_fn) (const char *, struct service *);
	int (*dump_fn) (const struct service *, bool, FILE *);
};

void message(const char *format, ...);
void message_e(const char *format, ...);
int service_add3 (char *keyword, char *lang, char *value, struct service *service);
int service_add2 (char *keywordlang, char *value, struct service *service);
int service_add1 (char *s, struct service *service);
int service_remove2 (char *keyword, char *lang, struct service *service);
int service_remove1 (char *keywordlang, struct service *service);
int service_read_default (const char *servicename, struct service *service);
int service_read_custom (const char *servicename, struct service *service);
int service_read (const char *servicename, struct service *service);
int service_dump (const struct service *service, bool all, FILE *f);
int services_read_list (struct services *services);
int services_dump_list (const struct services *services, FILE *f);
int use_current_language (void);
int service_start (const struct service *service);
int service_stop (const struct service *service);
int service_restart (const struct service *service);
int service_reload (const struct service *service);
int service_force_reload (const struct service *service);
int service_status (const struct service *service);

#endif /* SERVICE_H */
